#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
from numpy.linalg import norm

# Programm 7.1a Test auf positive Definitheit
# input:  Matrix a
# output: 1 -> pos. def., 0 -> nicht pos. def.
# function [flag]=testpd_gb(a);

def testpd_gb(a): 
    n = len(a)
    at = np.transpose(a)
    for i in range(0,n):
        for k in range(0,n):
            if (a[i,k] != at[k,i]):
                s = 0
                print('Matrix ist nicht symmetrisch')
                break
            else:
                s = 1
                continue
#           endif
#       endfor
#   endfor
    ew,ev = np.linalg.eig(a)
#   print(ew)
#   temp = eig(a+a');
    for i in range(0,n):
        re = ew[i].real
        im = ew[i].imag
        if (im != 0):
            print('Matrix hat komplexe Eigenwerte')
            s = 0
            break
#       endif        
#       im = eig(i).imag
        if (re <= 0.000000000001):
            print('Matrix ist nicht positiv definit')
            s = 0
            break
        else:
            continue
#       endif
#   endfor
# Man kann natuerlich den Test auf pos. Definitheit ueber
# eine Cholesky-Zerlegung machen, existiert diese, ist die Matrix
# pos. definit
    return s
#endfunction

# Programm 7.7 CG-Verfahren zur Loesung von A x = b , A symm., pos. def.
# gegeben A = (a_{ij}), b = (b_i) als Spaltenvektor, i,j=1,...,n
# input:  Matrix a = np.array([[..],...]), Vektor b = np.array([1,...])
# output: Loesung x
# Aufruf: x = cgit_gb(a,b);

def cgit_gb(a,b):
    n = len(b)
    x = np.array((n,n))
    z = np.array((n,n))
    ra = np.zeros(n) # ra = np.array((n,n))
#
    if (testpd_gb(a) != 1):
        return
#   endif
    y = b; x = y
    r = np.dot(a,x) - b
 #   
    p = -r
    it = 0
    while ( norm(r) > 1.0e-06 and it < 5 ):
        if (it > 0):
            e = np.dot(r.T,r)/np.dot(ra.T,ra) # e = np.dot(r,r.T)/np.dot(ra.T,ra)
            p = -r + e*p # p = -r + np.dot(e,p)
#       endif
        z = np.dot(a,p)
        q = np.dot(r.T,r)/np.dot(p.T,z)
        x = x + q*p # x = x + np.dot(q,p)
        ra = r
        r = r + q*z # r = r + np.dot(q,z)
        it = it + 1
#   endwhile
    return (x,it)
# endfunction

def main():
    a = np.array([[8,2,1,1],[2,8,2,1],[1,2,8,2],[1,1,2,8]])
    b = np.array([[1],[1],[1],[1]])

    x,it = cgit_gb(a,b)

    print('Loesung:',x)
    print('Iterationszahl:',it)

    print('Probe:',np.dot(a,x))

if __name__ == '__main__':
    main()

