#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import time
import matplotlib.pyplot as plt
from ito_integral_gb import ito_integral_gb

# Programm 10.5 em_milstein_allgemein_vs_exakt_gb.m ; 
# Euler-Maruyama und Milstein-Methode zur numerischen Loesung von
# SDE: dX = (alpha + mue*X) dt + (eta + sigma*X) dW, X(0) = Xzero ,
# alpha = 5 , mue = -2 , eta = 4 , sigma = 1 und Xzero = 1 .
#
# im Vergleich zur exakten Loesung
#
# Wiener-Prozess ueber [0,1], dt = 2^(-8).
# Euler-Maruyama, Milstein auf einem Gitter mit Dt = R*dt 

np.random.seed(50) # random seeding
alpha = 5 ; mue = -2 ; eta = 4 ; sigm = 1 ; Xzero = 1 # Problemparameter
T = 1 ; N = int(math.pow(2,10)) ; dt = 1/N
dW = np.zeros(N); W = np.zeros(N) # Pre-Allokierung
dW = np.dot(math.sqrt(dt),np.random.randn(N))  # Inkremente
W = np.cumsum(dW)  # approximierter Pfad eines Wiener-Prozess
ct2 = np.zeros(N)
Xtrue = np.zeros(N)
Xem = np.zeros(N)
Yem = np.zeros(N)
xh = np.zeros(N+1);
start = time.time()
# Aufbau der exakten Loesung
for j in range(0,N): # = 1:N
    ct1 = 0
    for k in range(0,j):
        summa = math.exp( (0.5*sigm*sigm - mue)*(k+1)*dt - sigm*W[k] )
        ct1 = ct1 + summa*dt
        ct2[k] = math.exp( (0.5*sigm*sigm - mue)*(k+1)*dt - sigm*W[k] )
#   endfor
#    wito = ito_integral_gb(ct2[0:j+1],dW[0:j+1],j+1)
    wito = ito_integral_gb(ct2[0:j],dW[0:j],j)        
    Xtrue[j] = ( Xzero+math.exp(sigm*W[0])*( (alpha-eta*sigm)*ct1+eta*wito ) ) \
        *math.exp( (mue-0.5*sigm*sigm)*(j+1)*dt+sigm*(W[j]-W[0]) )
# endfor 
tt = np.zeros(N+1)
for i in range(N+1):
    tt[i] = i*dt
# endfor
xh[0] = Xzero
for i in range(N):
    xh[i+1] = Xtrue[i]
# endfor
fig = plt.figure(1)
plt.plot(tt, xh, 'g-') # hold on
plt.xlabel('t')
plt.ylabel('Xtrue')
#
R = 4; Dt = R*dt; L = int(N/R)
xh1 = np.zeros(L+1); xh2 = np.zeros(L+1)
Xem = np.zeros(L); Yem = np.zeros(L) # Pre-Allokierung
Xtemp = Xzero
Ytemp = Xzero
for j in range(L):
    Winc =  sum(dW[R*j:R*(j+1)])
    Xtemp = Xtemp + Dt*(alpha+mue*Xtemp)+(eta+sigm*Xtemp)*Winc
    Xem[j] = Xtemp   #   Euler-Maruyama-Loesung
    Ytemp = Ytemp+Dt*(alpha+mue*Ytemp)+(eta+sigm*Ytemp)*Winc \
        +0.5*sigm*(eta+sigm*Ytemp)*(Winc*Winc-Dt)
    Yem[j] = Ytemp    #   Milstein-Loesung
# endfor
ende = time.time()
print('{:10.8f}s'.format(ende-start))
###
ttt = np.zeros(L+1)
for i in range(L+1):
    ttt[i] = i*Dt
xh1[0] = Xzero
for i in range(L):
    xh1[i+1] = Xem[i]
# endfor
xh2[0] = Xzero
for i in range(L):
    xh2[i+1] = Yem[i]
# endfor
plt.plot(ttt, xh1, 'b-') 
fig = plt.figure(2)
plt.plot(tt, xh, 'g-') # hold on
plt.plot(ttt, xh2, 'r-') 
plt.xlabel('t')
plt.ylabel('Xtrue')
eerrx = abs(Xem[L-1]-Xtrue[N-1])
merry = abs(Yem[L-1]-Xtrue[N-1])
# Programmende
