#!/usr/bin/python
# -*-coding: utf-8-*-

import math
import numpy as np

# Programm 4.1 zur Transfomation von a auf Hessenberg-Form
# input:  Matrix a0=(a0_{ij}) vom Typ n x n
# output: Hessenberg-Matrix h, orthogonale Matrix q^T
# Aufruf: [h,qt] = hessenberg_gb0(a)
# function [h,qt] = hessenberg_gb0(a0);

def hessenberg_gb0(a0):
    n = len(a0)
#
    a = np.zeros((n,2*n)) # a = [[0 for col in range(col)] for row in range(row)]
#    
    h = np.zeros((n,n)) # h = [[0 for col in range(col)] for row in range(row)]
    qt = np.zeros((n,n)) #qt = [[0 for col in range(col)] for row in range(row)]
#
    a[:,0:n] = a0[:,0:n]
# Aufbau des erweiterten Schemas [A|E]
# Endergebnis [H|Q^T]
    for i in range(0,n): # i=1:n
        for j in range(0,n): # j=1:n
            if (i == j):
                a[i,n+j]=1
            else:
                a[i,n+j]=0
#           endif
#       endfor
#   endfor
# Erzeugung des unteren Null-Dreiecks
    for j in range(0,n-2): # j=1:n-2
        summe = np.dot(a[j+1:n,j],a[j+1:n,j])
        c = np.sign(a[j+1,j])*math.sqrt(summe)
        if (c != 0):
            a[j+1,j]=c+a[j+1,j]
            d=1/(c*a[j+1,j]);
# Linksmultiplikation mit H_j
            for k in range(j+1,2*n): #   k=j+1:2*n
                summe = np.dot(a[j+1:n,j],a[j+1:n,k])
                e=d*summe
                a[j+1:n,k] = a[j+1:n,k] - e*a[j+1:n,j]
#           endfor
# Rechtsmultiplikation mit H_j
            for i in range(0,n): # i=1:n
                summe = np.dot(a[i,j+1:n],a[j+1:n,j])
                e=d*summe;
                a[i,j+1:n] = a[i,j+1:n] - e*a[j+1:n,j]
#           endfor
# update von a(j+1,j)...a(n,j)
            a[j+1,j]=-c
            a[j+2:n,j] = 0
#       endif
#   endfor
    h[:,0:n] = a[:,0:n]
    qt[:,0:n] = a[:,n:2*n]
    return (h, qt)
#endfunction

def main():
    a = np.array([[8,2,1,1],[2,8,2,1],[1,2,8,2],[1,1,2,8]])

    h, qt = hessenberg_gb0(a)

    print('Hessenbergmatrix:')
    print(h)
    print('orthogonale Matrix Q^T:')
    print(qt)

if __name__ == '__main__':
    main()

