#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import matplotlib.pyplot as plt

# Programm zur Bereitstellung 
# der rechten Differentialgleichungsseite f(x,y)
def f(x,y):
    y = y/(1.0 + x) + (x+1)*math.cos(x)
    return y
# endfunction

# Programm 8.2 (heun2rk3_gb.py)
# zur Loesung von gew. DGL 1. Ordnung y'=f(x,y), y(x_0)=y_0
# Heun-Verfahren 2. Ordnung eingebettet in ein RK-Verfahren 3. Ordnung
# input:  Start-Schrittweite h, Intervallgrenzen x0,x1, Anfangswert y0=y(x0),
#         Genauigkeit epsilon 
# output: Loesung y(x) an den Punkten x
# benutztes Programm: Nutzerfunktion f (rechte Seite der Dgl.)
# Aufruf: [y,x] = heun2rk3_gb(x0,x1,y0,h,epsilon)

def heun2rk3_gb(x0,x1,y0,h,epsilon):
    n = 10000
    x = np.zeros(n); y = np.zeros(n)
    hs = np.zeros(n)
    hs[0] = h
    yn = y0; y[0] = y0; xn = x0; x[0] = x0
    k = 0
    while (xn + h <= x1):
        k1 = f(xn,yn)
        k2 = f(xn+h,yn+h*k1)
        k3 = f(xn+0.5*h,yn+0.25*h*(k1+k2))
        yn = yn + 0.5*h*(k1+k2)
        rk = np.abs(h*(2*k3-k1-k2)/3)
        if (rk > epsilon):
            h = 0.5*h
#       endif
        if (rk < 0.1*epsilon):
            h = 2*h
#       endif
        xn=xn+h; x[k+1]=xn; y[k+1]=yn; hs[k+1]=h
        k=k+1
        if (k > n-2):
            print('n muss vergroessert werden')
            return (0,0)
#       endif
#   endwhile
    xx = x[0:k+1]; yy = y[0:k+1]
    return (yy,xx)
# endfunction

def main():
    x0 = 0
    x1 = 10
    y0 = 1
    h = 0.01
    epsilon = 1.0e-05
    yy,xx = heun2rk3_gb(x0,x1,y0,h,epsilon)

    import matplotlib.pyplot as plt
    plt.plot(xx,yy,color="blue",linewidth = 1, linestyle="-")

if __name__ == '__main__':
    main()

