#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math

# Programm 3.1 zur QR-Zerlegung einer Matrix c0 = q*r
# Erzeugung der wesentlichen Informationen fuer 
# die orthogonale Matrix Q und der oberen Dreiecksmatrix R
# input:  Matrix c=(c(ij)) vom Typ m x p gegeben (m >= p)
# output: Matrix c mit den wesentlichen Informationen
#         der orthogonalen Matrix q und der oberen Dreiecksmatrix r,
#         Erzeugung des inputs fuer das Programm bestapprqr_gb.m,
# Aufruf: [c,t] = qr_gb0(c);

def qr_gb0(c0):
    m = len(c0)
    p = len(c0[0])
    c = np.zeros((m,p))
    t = np.zeros(p)
    c[:,:] = c0[:,:]
    for l in range(0,p): # mslice[1:p]:
        gamma = np.dot(c[l:m,l],c[l:m,l])
        gamma = math.sqrt(gamma)
        if (c[l,l] < 0):
            gamma = -gamma
#       endif
        peta = math.sqrt(2*gamma*(gamma + c[l,l]))
        t[l] = -gamma #        t(l).lvalue = -gamma
        c[l,l] = (c[l,l] + gamma)/peta #        c(l, l).lvalue = (c(l, l) + gamma) / peta
        c[l+1:m,l] = c[l+1:m,l]/peta
        for j in range(l+1,p): # mslice[l + 1:p]:
            g = np.dot(c[l:m,l],c[l:m,j])
            g = g + g
            c[l:m,j] = c[l:m,j] - g*c[l:m,l]
#       endfor
#   endfor
    return (c, t)
#   endfunction

# Programm 3.2 zur Bestimmung der Bestapproximation x
# zur Behandlung des ueberbestimmten Systems C x = y
# unter Nutzung der im Programm 3.1 (qr_gb.py)  erzeugten 
# Matrix c = (c(ij)) und dem Vektor t(i) (QR-Zerlegung)
# input:  Matrix c, Vektor t als Ergebnis des Programms qr_gb0,
#         Vektor y als rechte Seite
# output: Loesung x, Residuenvektor d
# Aufruf: [x,d] = bestapprqr_gb0(c,t,y);

def bestapprqr_gb0(c, t, y0):
    m = len(c) # length(c(mslice[:], 1))
    p = len(c[0]) # length(c(1, mslice[:]))
    x = np.zeros(p) # list(range(0,p))
    d = np.zeros(m) # list(range(0,m))
    y = np.zeros(m)
    for i in range(m):
        y[i] = y0[i]
# Aufbau der rechten Seite Q^T y
    for l in range(0,p): # mslice[1:p]:
        summe = np.dot(c[l:m,l],y[l:m])
        summe = summe + summe
        y[l:m] = y[l:m] - summe*c[l:m,l]
#   endfor
#   Loesung des Gleichungssystems R x = Q^T y
    for l in range(0,p): # mslice[p:-1:1]:
        i = p-1-l
        summe = y[i]
        d[i] = 0 # d(i).lvalue = 0
        if ((i + 1) <= p):
            for k in range(i+1,p): # mslice[i + 1:p]:
                summe = summe - c[i,k]*x[k]
#           endfor
#       endif
        x[i] = summe/t[i] # x(i).lvalue = summe / t(i)
#   endfor
    if ((p + 1) <= m):
        d[p:m] = y[p:m]
#   endif
#   Ruecktransformation des Residuums
    for i in range(0,p): # mslice[p:-1:1]:
        l = p-i-1
        summe = np.dot(c[l:m,l],d[l:m])
        summe = summe + summe
        d[l:m] = d[l:m] - summe*c[l:m,l]
#   endfor
#    x
#    d
    return (x,d)
#   endfunction


def main():
    a = np.array([[1,2,3],[2,4,8],[3,5,7],[4,6,9]])
    c,t = qr_gb0(a)
    y = np.array([[1],[2],[1],[4]])

    x,d = bestapprqr_gb0(c,t,y)
    print('Loesung:',x)
    print('Defektvektor, Defekt-Norm:',d,np.linalg.norm(d))

if __name__ == '__main__':
    main()

