#!/usr/bin/python
# -*-coding: utf-8-*-

# Programm 2.1 zur LR-Zerlegung einer Matrix A
# input:  Matrix a0
# output: Matrix a, enthaelt im oberen Dreieck die Matrix R
#         und unterhalb der Hauptdiagonale die Matrix L       
#         Buchhaltungsvektor z der Zeilenvertauschungen,
#         output ist input fuer das Programm lrloes_gb
# Aufruf: [a,z] = lr_gb0(a0);

import numpy as np

def lr_gb0(a0):
    n = len(a0) # n = length(a(mslice[:], 1))
    a = np.zeros((n,n))
    a[:,:] = a0[:,:]
    z = np.zeros(n) # list(range(0,n))
    z = np.linspace(0,n-1,n)
    a0[:,:] = a[:,:]
#   'Elimination'
    for j in range(0,n - 1):
#       'Pivotsuche'
        apivot = abs(a[j,j])
        p = j
        for l in range(j + 1,n):
            if (abs(a[l,j]) > abs(apivot)):
                apivot = a[l,j]
                p = l
#           endif
#       endfor
        if (apivot != 0):
            for i in range(1,n):
                c = a[j,i]
                a[j,i] = a[p,i]
                a[p,i] = c
#           endfor
            pp = z[j]
            z[j] = z[p]  # z(j).lvalue = z(p)
            z[p] = pp    # z(p).lvalue = pp
#           'Elimination'
            for i in range(j+1,n):  #  mslice[j+1:n]:
                a[i,j] = a[i,j]/a[j,j] # a(i,j).lvalue = a(i,j)/a(j,j)
                a[i,j+1:n] = a[i,j+1:n] - a[i,j]*a[j,j+1:n]
#           endfor
#       endif
#   endfor

    idet = 1
    for j in range(0,n):
        if (abs(a[j,j]) < 1.0e-10):
            idet = 0
#       endif
#   endfor
    if (idet == 0):
        print('Matrix singulaer, lrloes_gb nicht anwendbar')
#   endif
    return (a, z)
# endfunction

# Programm 2.2
# Loesung des linearen Gleichungssystems lr x = b
# input:  Matrix a (enthaelt l,r) und Buchhaltungsvektor z
#         als Ergebnis von Programm lr_gb
#         Spaltenvektor b als rechte Seite
# output: Loesung x
# Aufruf: x = lrloes_gb(a,b,z)

def lrloes_gb(a, b, z):
    n = len(a)
    y = np.zeros(n); x = np.zeros(n)
    for i in range(0,n):
        y[i] = b[int(z[i])]
    b[:,0] = y[:]
#   Vorwaertseinsetzen
    for i in range(0,n):
        y[i] = b[i]  #  y(i).lvalue = b(i)
        for j in range(0,i): # for j=1:i-1 ! Achtung
            y[i] = y[i] - a[i,j]*y[j]  #  y(i).lvalue = y(i) - a(i, j) * y(j)
#       endfor
#   endfor
#   Zweiter("Schritt")
    x[n-1] = y[n-1]/a[n-1,n-1]  #  x(n).lvalue = y(n)/a(n, n)
    for l in range(0,n-1):  #  mslice[n - 1:-1:1]:
        k = n-2-l
        c = y[k]
        for j in range(k+1,n):  #  mslice[k + 1:n]:
            c = c - a[k,j]*x[j]
#       endfor
        x[k] = c/a[k,k]  #  x(k).lvalue = c / a(k, k)
#   endfor
    return x  #   x = x.cT
#   endfunction


def main():
    a = np.array([[8,2,1,1],[2,8,2,1],[1,2,8,2],[1,1,2,8]])

    a0,z0 = lr_gb0(a)

    b = np.array([[1],[1],[1],[1]])

    x = lrloes_gb(a0,b,z0)

    print('Probe:',np.dot(a,x))

if __name__ == '__main__':
    main()

