#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import scipy.optimize as scopt
import math
import matplotlib.pyplot as plt

# Programm zur Bereitstellung 
# der Funktion f(y,x,t) der
# DAE f(x',x,t) = 0.
def fo(y,x,t):
    f1 = y[0] - x[1]
    f2 = y[1] + math.sin(x[0])
    return np.array([f1,f2])
# endfunction

# Programm 9.1  (bdf2o.py)
# zur Loesung von gew. DGL 1. Ordnung f(x',x,t)=0, x(t0)=x0
# 2-schrittiges BDF-Verfahren (Ordnung 2)
# input:  Schrittweite h, Intervallgrenzen t0,tf,
#         Anfangswerte x0=x(t0), x1=x(t0+h)
# output: Loesung x(t) an den Punkten t
# benutztes Programm: Nutzerfunktion f (DAE Funktion)
# Aufruf: [x,t] = bdf2o(t0,tf,x0,x1,h);

def bdf2o(t0,tf,x0,x1,h):
    n = int((tf - t0)/h)
    m = len(x0)

    t = np.zeros(n+1); x = np.zeros((n+1,m))
    xk = np.zeros(m);
    xkm1 = np.zeros(m)
    t[0] = t0; t[1] = t0+h
    x[0] = x0; x[1] = x1
    xkm1 = x0 # starte mit x_{k-1}:=x0
    xk = x1   # und x_{k}:=x1
    tkp1 = t0+2.0*h # neuer Zeitpunkt t_{k+1}

    # Die Loesung xkp1=x_{k+1} eines BDF-Schrittes
    # ist die Nullstelle von bdf2step_f (nahe x_{k}).
    def bdf2step_fo(x,xk,xkm1,tkp1,h):
        ykp1 = (1.5*x-2.0*xk+0.5*xkm1)/h
        return fo(ykp1,x,tkp1)

    # berechne Nullstelle von bdf2step_f
    # und nehme x_{k} als Startnaeherung:

    for k in range(1,n):
        args = (xk,xkm1,tkp1,h)
        xkp1 = scopt.fsolve(bdf2step_fo,xk,args)

        t[k+1] = tkp1   # speichere Zeitpunkt t_{k+1}
        x[k+1,:] = xkp1 # speichere Loesung x_{k+1}
        tkp1 = tkp1+h   # update t_{k+1} fuer naechsten Schritt
        xkm1 = xk       # update x_{k-1} fuer naechsten Schritt
        xk = xkp1       # update x_{k} fuer naechsten Schritt
    return (x,t)
# endfunction

def main():
    t0 = 0
    tf = 2*math.pi
    h = 1.0e-2
    x0 = np.array([1.0,0.0])
    x1 = np.array([math.cos(h),math.sin(h)])
    
    x,t = bdf2o(t0,tf,x0,x1,h)

    # berechne exakte Lösung
    nn = len(t)
    xx = np.ones((nn,len(x0)))
    for i in range(nn):
        xx[i,1] = -math.sin(t[i])
        xx[i,0] = math.cos(t[i])


    plt.figure(1)
    plt.plot(t, x[:,0], color="blue", linewidth=1,linestyle="-")
# Vergleich mit der exakten Loesung
    plt.plot(t, xx[:,0], color="red", linewidth=1,linestyle="--")

    plt.legend(["num. Loesung", "exakte Loesung"], loc ="upper right")

    plt.savefig("schwingung.png",dpi=300, bbox_inches='tight')
    plt.show()


if __name__ == '__main__':
    main()

