#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math
import time
import matplotlib.pyplot as plt
from numpy.linalg import norm

# Programm 10.7 Test der starken Konvergenz des Milstein-Verfahrens
# am Beispiel dX = r *X*(K -X) dt + eta*X dW, X(0) = Xzero ,
# r = 2 , K= 1 , eta = 1 and Xzero = 0.5 .
#
# Wiener-Prozess-Pfad ueber [0,1], h = 2^(-11).
# Milstein Schritte 128*dt , 64*h , 32*h , 16*h (auch h zum Vergleich)
#
# Test der starken Konvergenz bei T=1: E | X_L - X(T) | 
# vektorisiert, alle Pfade werden simultan berechnet

np.random.seed(100) # random seeding

r = 2 ; K = 1 ; eta = 0.25 ; Xzero = 0.5 
T = 1 ; N = int(math.pow(2,11)) ; h = T/N
M = 500 ; # Zahl der gesampleten Pfade
R = np.array([ 1 , 16 , 32 , 64 , 128 ])  # Milstein-Schritte are R*h
dW = np.zeros((M,N))  # Pre-Allokierung
dW = np.dot(math.sqrt(h),np.random.randn(M,N)) # dW = sqrt(h)*randn(M,N); # Inkremete
Xmil = np.zeros((M,5))  # Allokierung
start = time.time()
for p in range(5):
    Dt = R[p]*h; L = int(N/R[p])  # L Milstein-Schritte Dt = R h
    Xtemp = Xzero*np.ones(M) 
    for j in range(L):
        B = dW[:,R[p]*j:R[p]*(j+1)]
        Winc = np.sum(B, axis = 1)
        Xtemp = Xtemp + Dt*r*(Xtemp*(K-Xtemp)) + \
            eta*(Xtemp*Winc)    \
            + 0.5*eta*eta*(Xtemp*(Winc*Winc - Dt));
#   endfor
    Xmil[:,p] = Xtemp # Milsteinloesung bei t =1
# endfor
Xref = Xmil[:,0]; # Referenzloesung
Xref4 = np.zeros((M,4))
for j in range(M):
    Xref4[j,0] = Xref[j]
    Xref4[j,1] = Xref[j]
    Xref4[j,2] = Xref[j]
    Xref4[j,3] = Xref[j]
#   endfor
Xerr = abs(Xmil[:,1:5] - Xref4)
xerrm = np.mean(Xerr,axis=0) # mean(Xerr); # mittlere Pfad-weise Fehler
#
ende = time.time()
print('{:10.8f}s'.format(ende-start))
##
Dtvals = h*R[1:5] # np.array([1,2,4,8,16]) # Dtvals = h*(2.^([0:4])) ;
Dtvalsq = np.zeros(4)
rhs = np.zeros(4)
plt.loglog(Dtvals, xerrm, '*')
plt.loglog(Dtvals, Dtvals, 'g-') # Referenzsteigung von 1
plt.xlabel('h')
plt.ylabel('Mittel von |X(T) - X_L|')

A = np.array([[1, math.log(Dtvals[0])],[1,math.log(Dtvals[1])], \
               [1,math.log(Dtvals[2])], \
        [1,math.log(Dtvals[3])]])
for i in range(4):
    rhs[i] = math.log(xerrm[i])
# endfor    
# Direct least square regression
sol = np.dot((np.dot(np.linalg.inv(np.dot(A.T,A)),A.T)),rhs)
print(sol)
q = sol[1]
C = math.exp(sol[0])
for j in range(4):
    Dtvalsq[j] = C*math.pow(Dtvals[j],q)
# endfor
# tatsaechliche Steigung
plt.loglog(Dtvals, Dtvalsq, 'r-')
resid = norm(np.dot(A,sol) - rhs)
# Programmende
