#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import math

# Programm 9.3 zur Loesung eines Anfangs-Randwertproblems (parapde_gb.py)
#  u_t - u_xx = 5x(1-x), 0<x<1, 0<t <=10, u(0,t)=u(1,t)=0, u(x,0)=sin(2x\pi)
# mit der FEM (raeumlich, stueckweise lineare Elemente), 
# Zeitintegration: Euler-rueckwaerts
# n=21, h=1/21, tau=0,1, m=10/tau Zeitschrittzahl
# input:
# output: Loesung yp(x,t) auf dem Raum-Zeit-Gitter x1,x2
# Aufruf: [yp,x1,x2] = parapde_gb0;

def parapde_gb0():
    r = np.zeros(20); r0 = np.zeros(20); x = np.zeros(20)
    h = 1/21; tau = 0.1
    m = int(10/tau)
    x = np.linspace(h,20*h,20)
    amasse = np.zeros( (20,20) ) # amasse = [[0 for j in range(20)] for i in range(20)]
    asteif = np.zeros( (20,20) ) # asteif = [[0 for j in range(20)] for i in range(20)]
    y = np.zeros((20,m)) # y = [[0 for j in range(m)] for i in range(20)]
    amasse = amasse + 4*np.eye(20)
    asteif = asteif + 2*np.eye(20)
    for k in range(19):
        amasse[k+1,k]=1; amasse[k,k+1]=1; asteif[k+1,k]=-1; asteif[k,k+1]=-1
#   endfor
    amasse = np.dot(h/6,amasse)
    asteif = np.dot(1/h,asteif)
    r[:] = 5*x[:]*(1 - x[:])*h/2
    for k in range(20):
        r0[k]= math.sin(x[k]*2*math.pi)*h/2
#   endfor
    ams = np.linalg.solve(amasse,asteif)
#
    rm = np.linalg.solve(amasse,r)
    y0 = np.linalg.solve(amasse,r0)
    mit = (1/tau)*np.eye(20)+ams
# Euler-Verfahren
    yn = y0
    for k in range(m):
        yn = np.linalg.solve(mit,(yn/tau+rm))
        for j in range(20):
            y[j,k] = yn[j]
#       endfor
#   endfor
# Aufbereitung der Ergebnisse fuer die Grafik
    yp = np.zeros( (22,m+2) ) # y = [[0 for j in range(m)] for i in range(20)]
#    
    yp[1:21,0] = y0
    yp[1:21,1:m+1] = y
    x1 = np.zeros(22)
    x2 = np.zeros(m+2)
    x1[0:20] = x
    x1[20] = x1[19] + h
    x1[21] = x1[20] + h
    for j in range(m+2):
        x2[j] = j*tau
#   endfor 
    return (y,yp,x1,x2)
# endfunction

def main():
    y,yp,x1,x2 = parapde_gb0()

    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D

    X, Y = np.meshgrid(x1, x2, indexing='ij')
    fig = plt.figure()
    axes = plt.axes(projection='3d')

    Z = yp
    plt.xlabel('x1')
    plt.ylabel('x2')

    axes.set_title('Surface plot of pde solution')
    axes.plot_surface(X, Y, Z)

    plt.show()

if __name__ == '__main__':
    main()

