#!/usr/bin/python
# -*-coding: utf-8-*-

import math
import numpy as np

# Programm 4.1 zur Transfomation von a auf Hessenberg-Form
# input:  Matrix a0=(a0_{ij}) vom Typ n x n
# output: Hessenberg-Matrix h, orthogonale Matrix q^T
# Aufruf: [h,qt] = hessenberg_gb0(a)
# function [h,qt] = hessenberg_gb0(a0);

def hessenberg_gb0(a0):
    n = len(a0)
#
    a = np.zeros((n,2*n)) # a = [[0 for col in range(col)] for row in range(row)]
#
    h = np.zeros((n,n)) # h = [[0 for col in range(col)] for row in range(row)]
    qt = np.zeros((n,n)) #qt = [[0 for col in range(col)] for row in range(row)]
#
    a[:,0:n] = a0[:,0:n]
# Aufbau des erweiterten Schemas [A|E]
# Endergebnis [H|Q^T]
    for i in range(0,n): # i=1:n
        for j in range(0,n): # j=1:n
            if (i == j):
                a[i,n+j]=1
            else:
                a[i,n+j]=0
#           endif
#       endfor
#   endfor
# Erzeugung des unteren Null-Dreiecks
    for j in range(0,n-2): # j=1:n-2
        summe = np.dot(a[j+1:n,j],a[j+1:n,j])
        c = np.sign(a[j+1,j])*math.sqrt(summe)
        if (c != 0):
            a[j+1,j]=c+a[j+1,j]
            d=1/(c*a[j+1,j]);
# Linksmultiplikation mit H_j
            for k in range(j+1,2*n): #   k=j+1:2*n
                summe = np.dot(a[j+1:n,j],a[j+1:n,k])
                e=d*summe
                a[j+1:n,k] = a[j+1:n,k] - e*a[j+1:n,j]
#           endfor
# Rechtsmultiplikation mit H_j
            for i in range(0,n): # i=1:n
                summe = np.dot(a[i,j+1:n],a[j+1:n,j])
                e=d*summe;
                a[i,j+1:n] = a[i,j+1:n] - e*a[j+1:n,j]
#           endfor
# update von a(j+1,j)...a(n,j)
            a[j+1,j]=-c
            a[j+2:n,j] = 0
#       endif
#   endfor
    h[:,0:n] = a[:,0:n]
    qt[:,0:n] = a[:,n:2*n]
    return (h, qt)
#endfunction

# Programm 4.2 Ein Schritt H^k,P^k --> H^{k+1},P^{k+1} zum QR-Verfahren 
# sukzessive Berechnung der Eigenwerte einer (n x n)-Hessenberg-Matrix H
# H^k,P^k Shift kappa und delta (kleinste Zahl >0) sind vorgegeben
# input:  h0=H^k, p0=P^k (P^0 = eye), Shift kappa und delta (kleinste Zahl >0)
# output: h=H^{k+1}, p=P^{k+1}
# Aufruf: [h,p] = qriteration_gb0(h0,p0,kappa,delta);

def qriteration_gb0(h0,p0,kappa,delta):
    n = len(h0)
    h = np.zeros((n,n)) ; p = np.zeros((n,n))
    h[:,:] = h0[:,:]
    p[:,:] = p0[:,:]
    ss = 1; cs = 1
    h[0,0] = h[0,0] - kappa
    for i in range(0,n): #  i=1:n
        if (i < n-1):   # i < n
            if (abs(h[i,i]) < delta*abs(h[i+1,i])):
                w= abs(h[i+1,i])
                c=0
                s = np.sign(h[i+1,i])
            else:
                w=math.sqrt(h[i,i]*h[i,i] + h[i+1,i]*h[i+1,i])
                c = h[i,i]/w;
                s = -h[i+1,i]/w;
#           endif
            h[i,i]=w;
            h[i+1,i]=0;
            h[i+1,i+1]=h[i+1,i+1]-kappa;
            for j in range(i+1,n): # j=i+1:n
                g=c*h[i,j]-s*h[i+1,j]
                h[i+1,j]=s*h[i,j]+c*h[i+1,j]
                h[i,j]=g
#           endfor
#       endif
        if (i > 0):
            for j in range(0,i+1): # j=1:i
                g=cs*h[j,i-1]-ss*h[j,i]
                h[j,i]=ss*h[j,i-1]+cs*h[j,i]
                h[j,i-1]=g
#           endfor
            h[i-1,i-1]=h[i-1,i-1]+kappa
#       endif
        if (i < n-1):
            for j in range(0,n): #  j=1:n
                g=c*p[j,i]-s*p[j,i+1]
                p[j,i+1]=s*p[j,i]+c*p[j,i+1]
                p[j,i]=g
#           endfor
#       endif[[0 for col in range(col)] for row in range(row)]
        cs = c; ss = s
#   endfor
    h[n-1,n-1]=h[n-1,n-1]+kappa
    return h,p
# endfunction


def main():
    a = np.array([[8,2,1,1],[2,8,2,1],[1,2,8,2],[1,1,2,8]])

    h, qt = hessenberg_gb0(a)

    p = np.eye(4)
    kappa = 1.5
    delta = 1.0e-06
    epsilon = 1.0
    it = 0

    while (epsilon > 1.0e-06 and it < 100 ):
        h,p = qriteration_gb0(h,p,kappa,delta)
        etol = np.array([h[1,0],h[2,1],h[3,2]])
        epsilon = np.linalg.norm(etol)
        print(epsilon)
        it = it + 1

    print('Grenzwert der Folge H_n:')
    print(h)
    print('it=',it)

if __name__ == '__main__':
    main()

