#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np

# Programm 8.4a
def laf(x):
    y = x
    return y
# endfunction

# Programm 8.4b
def kf(x):
    y = -4*x
    return y
# endfunction

# Programm 8.4c
def cf(x):
    y = 0*x
    return y
# endfunction

# Programm 2.4 zur Loesung 
# eines tridiagonalen linearen Gleichungssystems
#   -pc(k)*yps(k-1)+pd(k)*yps(k)-pe(k)*yps(k+1)=pf(k),k=1,n
# mit einer schwach diagonal dominanten Koeffizientenmatrix!
# input:
#   n - Endindex 
#   pc, pd, pe - Vektoren der Dimension l der tridiag. Matrix
#   pf         - Vektor der rechten Seiten
#   pc(1) und pe(n) sind dummy-Elemente
# output
#   yps        - Ergebnisvektor
# c und d sind Hilfsfelder
# aufruf: yps = tridia_gb(pc,pd,pe,pf,n);

def tridia_gb(pc,pd,pe,pf,n):
# function [yps]= tridia_gb(pc,pd,pe,pf,n);
    c = np.zeros(n)
    d = np.zeros(n)
    yps = np.zeros(n)
    c[0] = pe[0]/pd[0];
    for i in range(1,n-1): #  i=2:n-1
        c[i] = pe[i]/(pd[i] - c[i-1]*pc[i])
#   endfor
    d[0] = pf[0]/pd[0]
    for i in range(1,n): # i=2:n
        d[i] = (pf[i] + d[i-1]*pc[i])/(pd[i] - c[i-1]*pc[i]);
#   endfor
    yps[n-1] = d[n-1]
    for k in range(0,n-1): # i=n-1:-1:1
        i = n-k-2
        yps[i] = d[i] + c[i]*yps[i+1]
#   endfor
    return yps        
# endfunction


# Programm 8.4 zur Loesung eines Zweipunkt-Randwertproblmes (rwp2p2_gb.m)
# -(laf(x)y')' + kf(x)y = cf(x),\; y'(a)=eta0, y(b)=eta1
# vorzugeben: a, b, eta0, eta1, n
# input:  Intervallgrenzen a,b, Randwerte eta0, eta1, n=(b-a)/h
# output: Loesung yy(x) an den Stellen x
# Benutzung der Programme tridia_gb (Programm 2.4), laf, kf, cf
# Aufruf: [yy,x] = rwp2p2_gb1(a,b,eta0,eta1,n);

def rwp2p2_gb1(a,b,eta0,eta1,n):
    h = (b-a)/n
    dd = np.zeros(n-1); cc = np.zeros(n-1); ee = np.zeros(n-1)
    bb = np.zeros(n-1)
    y = np.zeros(n-1); yy = np.zeros(n+1)
    vlaf = np.zeros(n)
    vkf = np.zeros(n+1)
    vcf = np.zeros(n+1)
    x = np.linspace(a,b,n+1)
    xp = (x[:-1]+x[1:])/2
    dxp=x[1:]-x[:-1]
    dx=xp[1:]-xp[:-1] 
# Vektoren der Funktionswerte
    vlaf[:] = laf(xp[:])
    vkf[:] = kf(x[:])
    vcf[:] = cf(x[:])
#
    dd[0] = vlaf[1]/(dxp[1]*dx[0])+vkf[1]
    cc[0] = 0
#
    dd[1:n-1] = vlaf[2:n]/(dxp[2:n]*dx[1:n-1])+ vlaf[1:n-1]/(dxp[1:n-1]*dx[1:n-1])+ vkf[2:n]
    cc[1:n-1] = vlaf[1:n-1]/(dxp[1:n-1]*dx[1:n-1])
    ee[0:n-2] = vlaf[1:n-1]/(dxp[1:n-1]*dx[1:n-1])
    ee[n-2] = 0
    bb[0] = vcf[1] + eta1*vlaf[0]/dx[0]
    bb[1:n-2] = vcf[2:n-1]
    bb[n-2] = vcf[n-1]+eta1*vlaf[n-1]/(dxp[n-1]*dx[n-2])
# Loesung
    y = tridia_gb(cc,dd,ee,bb,n-1)
    yy[0] = y[0] - dxp[0]*eta0
    yy[1:n] = y[0:n-1]
    yy[n] = eta1
    return (yy,x)
# endfunction

def main():
    a=0
    b=8
    eta0=0
    eta1=-0.1749
    n=200
    y,x = rwp2p2_gb1(a,b,eta0,eta1,n)

    import matplotlib.pyplot as plt
    plt.plot(x,y,color="blue",linewidth = 1, linestyle="-")

if __name__ == '__main__':
    main()

