#!/usr/bin/python
# -*-coding: utf-8-*-

import time
import math
import numpy as np
import matplotlib.pyplot as plt

# function ui = uinit( x, ictype )
def uinit(x,ictype):
    xshift = 1.0
#   switch ictype    
    if (ictype == 1): # case 1  # shock (uL > uR)
        uL = 1.0; uR = 0.5; ui = uR + (uL-uR) * ((x-xshift) <= 0.0)
    if (ictype == 2): #  case 2    # expansion (uL < uR)
        uL = 0.5; uR = 1.0; ui = uR + (uL-uR) * ((x-xshift) <= 0.0)
# end # switch
    return ui
# endfunction

# Programm 9.4: slopelim_gb1.m, Loeser der Advektionsgleichung u_t + au_x = 0
# Methoden : 'Lax-Wendroff/lw' (Standard Lax-Wendroff), imeth = 1
#            'Lax-Wendroff-sl/lwsl' (Lax-Wendroff mit minmod-slope-Limiter), imeth = 2 
#            'upwind', imeth = 0
# Anfangsbedingung:  1 = stueckw. konst. (Shock), 2 = stueckw. konst. (Expansion)
# Beispielaufruf: [uall] = slopelim_gb1(1,100,0.01,1)
def slopelim_gb1(imeth, nx, dt, ictype ): # function slopelim_gb( method, nx, dt, ictype )
    a = 1.0
    tend = 4; xmax = 8     # Endzeit, raeumliches Intervall [0,xmax]
    dx = xmax/nx # R
#    Raumschrittweite
    x = np.zeros(nx+1)
    sigma = np.zeros(nx)
    u = np.zeros(nx+1)
    u0 = np.zeros(nx+1)
    unew = np.zeros(nx+1)
    for i in range(nx+1):
        x[i] = i*dx # x  = [0 : dx : xmax]; 
#   endfor
    nt = math.floor(tend/dt); dt = tend / nt
    ntprint = 50; # Zeitschrittabstand der Plots der Profile
# Anfangswert
    u0   = uinit(x,ictype); unew = 0*u0
    for j in range(nx+1):
        u[j] = u0[j]
#   endfor
    print('Method: ',imeth) # disp(['Method: ',method]);i
    print('dx =',dx) # disp([' dx=',num2str(dx)]);
    print('dt =',dt) # disp([' dt=',num2str(dt)]);
    ntprint = min(nt, ntprint); dtprint = tend / ntprint;
# Loesung im Zeit-Raum-Gebiet.
    uall = np.zeros( (ntprint+1,nx+1) )
    uall[0:nx+1] = u0[0:nx+1]
    ip = 1
    end = nx
    for i in range(nt): # = 1 : nt,
        t = (i+1)*dt
#        switch lower(method)
        if (imeth == 2): # case {'lax-wendroffsl','Lax-Wendroffsl'}
#  slopes - minmod-Limiter
            for j in range(1,nx): # =2:nx,
                dux = (u[j] -u[j-1])/dx; duxp = (u[j+1]-u[j])/dx
                if (abs(dux) < abs(duxp) and dux*duxp > 0): 
                    sigma[j] = dux
                elif (abs(dux) > abs(duxp) and dux*duxp > 0):
                    sigma[j] = duxp
                else:
                    sigma[j] = 0
#               endif
#           endfor
            sigma[0] = sigma[1]
            unew[1:nx] = u[1:nx] -a*dt/dx*(u[1:nx]-u[0:nx-1]) - a*dt/2*(sigma[1:nx]-sigma[0:nx-1]) \
                    + 0.5*(a*dt)*(a*dt)/dx*(sigma[1:nx] - sigma[0:nx-1])
            unew[0]   = u[0]; unew[end] = u[end]
        if (imeth == 1): # case {'lax-wendroff','Lax-Wendroff'}
#  slopes - Standard
            sigma[0:nx] = (u[1:nx+1] -u[0:nx])/dx
            unew[1:nx] = u[1:nx] -a*dt/dx*(u[1:nx]-u[0:nx-1]) - a*dt/2*(sigma[1:nx]-sigma[0:nx-1]) \
                    + 0.5*(a*dt)*(a*dt)/dx*(sigma[1:nx] - sigma[0:nx-1])
            unew[0]   = u[0]; unew[end] = u[end]
        if (imeth == 0): #   otherwise  # upwind (default)
            unew[1:nx] = u[1:nx] - a*dt/dx*(u[1:nx]-u[0:nx-1])
            unew[0]    = u[0]; unew[end] = u[end]
#  end  # switch
# Plot der Profile, alle ntprint Zeitschritte
        if (t >= ip*dtprint):
            plt.plot(x, unew)
            plt.xlabel('x')
            plt.ylabel('y')
            plt.title('Loesung zum Zeitpunkt t='+ str(t)) # ,'%9.4f'))
            time.sleep(0.1)
            plt.show()
            ip = ip + 1; uall[ip-1,:] = unew
#       endif
        u[0:nx+1] = unew[0:nx+1]
#   endfor
    return uall
#   endfor
# endfunction

def main():
    uall = slopelim_gb1(2,100,0.01,1)

if __name__ == '__main__':
    main()

