#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np
import time
### from F import F
### from G import G
### from gradF import gradF
### from gradG import gradG

# Programm 7.3 zum SQP-Verfahren (sqp_gb.m)
# Min F(x,y) , u.d.N. G(x,y) = 0
#
# F, G, gradF und gradG sind als Programme (Function)
# Spaltenvektoren, bereitzustellen
# input:  Startwert (x0,y0,lambda0), Lagrange-Multiplikator 
# output: Loesung (x*,y*,lambda*), F(x*,y*) und G(x*,y*)
# Aufruf: [xs,ys,lambda,f,g,k] = sqp_gb(x0,y0,lambda0)
# 

def G(x,y):
    g = x*x + y*y -1 
    return g
# endfunction

def F(x,y):
    f = x*y
    return f
# endfunction

def gradG(x,y):
    grad = np.zeros(2)
    grad[0] =2*x 
    grad[1] = 2*y
    return grad
# endfunction

def gradF(x,y):
    grad = np.zeros(2)
    grad[0] = y
    grad[1] = x
    return grad
# endfunction

def sqp_gb(x0,y0,lambda0):
    M = np.array([[0, 0, 0], [0, 0, 0], [0, 0, 0]])
    x = x0; y = y0; lambd = lambda0
    X0 = np.zeros(3)
    B = np.array([[2*lambd, 1], [1, 2*lambd]])
    gradg = gradG(x,y) ; gradf = gradF(x,y)

    M = [[B[0,0], B[0,1], gradg[0]], [B[1,0], B[1,1], gradg[1]], [gradg[0], gradg[1], 0.0]]
    rhs = [[-gradf[0] - lambd*gradg[0]], [-gradf[1] - lambd*gradg[1]], [-G(x,y)]]
#
    epsilon = 1.0e-06
    k = 0
#
    start = time.time()
#
    while (abs(G(x,y)) > epsilon and k < 20):
        k = k+1
        X = np.linalg.solve(M,rhs)
        for i in range(3):
            X0[i] = X[i]
        dx = X0[0]; dy = X0[1]; dlambda = X0[2]
        x = x + dx; y = y + dy; lambd = lambd + dlambda
        gradg = gradG(x,y) ; gradf = gradF(x,y)
        B = np.array([[2*lambd, 1], [1, 2*lambd]])
        M = [[B[0,0], B[0,1], gradg[0]],[B[1,0], B[1,1], gradg[1]],[gradg[0], gradg[1], 0.0]]
        rhs = [[-gradf[0] - lambd*gradg[0]], [-gradf[1] - lambd*gradg[1]], [-G(x,y)]]
#   endwhile
    ende = time.time()
    print('{:10.8f}s'.format(ende-start))
#
    xs = x; ys = y; lambdas = lambd
    f = F(x,y); g = G(x,y)
    return (xs,ys,lambdas,f,g,k);
# endfunction

def main():
    x0 = 0.3
    y0 = 0.3
    lambda0 = 0.9
    xs,ys,lambdas,f,g,k = sqp_gb(x0,y0,lambda0)

    print('Loesung:',xs,ys,lambdas)
    print('Iterationszahl:',k)

if __name__ == '__main__':
    main()

