#!/usr/bin/python
# -*-coding: utf-8-*-

import numpy as np

# Programm 2.4 zur Loesung 
# eines tridiagonalen linearen Gleichungssystems
#   -pc(k)*yps(k-1)+pd(k)*yps(k)-pe(k)*yps(k+1)=pf(k),k=1,n
# mit einer schwach diagonal dominanten Koeffizientenmatrix!
# input:
#   n - Dimension des Gleichungssystems
#   pc, pd, pe - Vektoren der Dimension l der tridiag. Matrix
#   pf         - Vektor der rechten Seiten
#   pc(1) und pe(n) sind dummy-Elemente
# output
#   yps        - Ergebnisvektor
# c und d sind Hilfsfelder
# aufruf: yps = tridia_gb(pc,pd,pe,pf,n);

def  tridia_gb(pc,pd,pe,pf,n):
# function [yps]= tridia_gb(pc,pd,pe,pf,n);
    c = np.zeros(n)
    d = np.zeros(n)
    yps = np.zeros(n)
    c[0] = pe[0]/pd[0];
    for i in range(1,n-1): #  i=2:n-1
        c[i] = pe[i]/(pd[i] - c[i-1]*pc[i])
#   endfor
    d[0] = pf[0]/pd[0]
    for i in range(1,n): # i=2:n
        d[i] = (pf[i] + d[i-1]*pc[i])/(pd[i] - c[i-1]*pc[i]);
#   endfor
    yps[n-1] = d[n-1]
    for k in range(0,n-1): # i=n-1:-1:1
        i = n-k-2
        yps[i] = d[i] + c[i]*yps[i+1]
#   endfor
    return yps
# endfunction

def main():
    l = -np.ones(5)
    d = 4*np.ones(5)
    u = -np.ones(5)
    b = np.ones(5)

    n = 5

    y = tridia_gb(l,d,u,b,n)

    print("y=",y)

if __name__ == '__main__':
    main()

